@@ -71,3 +71,10 @@
 
     #include <setjmp.h>
 #endif
+
+#ifndef CURL_STATICLIB
+#define CURL_STATICLIB
+#endif
+#ifndef STANDALONE
+#include "curl/curl.h"
+#endif
